/*
 * Java Danmaku Engine
 * Copyright (c) 2020-2020 Hell Hole Studios
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.hhs.jade.game.entity;

import com.hhs.jade.game.BossScene;
import com.hhs.jade.game.Entity;
import com.hhs.jade.game.Player;
import com.hhs.jade.util.Collision;
import com.hhs.jade.util.FlyingAnimation;
import com.hhs.jade.util.J;
import com.hhs.jade.util.U;
import com.badlogic.gdx.graphics.g2d.Batch;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.graphics.g2d.TextureRegion;

/**
 * Base class for enemies
 *
 * @author XGN
 */
public class Enemy extends Entity {

    public transient FlyingAnimation was;
    public float x, y;
    /**
     * The health point of the enemy
     */
    public double hp;
    /**
     * Hitbox radius for shot
     */
    public Collision.CollisionData collisionS;
    /**
     * Hitbox radius for players
     */
    public Collision.CollisionData collisionP;

    private float lastx;
    public boolean isBoss;

    /**
     * Auto remove when out of bound for too far
     */
    public boolean enableOOBCheck = true;

    public Enemy(TextureAtlas atlas, String regionName, int frameLength, int transitionFrameLength, float x, float y,
                 float hp, float radiusS, float radiusP, boolean isBoss) {
        super();
        was = new FlyingAnimation(atlas.findRegions(regionName + "_left"), atlas.findRegions(regionName + "_center"),
                atlas.findRegions(regionName + "_right"), atlas.findRegions(regionName + "_toLeft"),
                atlas.findRegions(regionName + "_toRight"), frameLength, transitionFrameLength);
        this.x = x;
        this.y = y;
        this.hp = hp;
        this.collisionS = new Collision.Circle(radiusS);
        this.collisionP = new Collision.Circle(radiusP);
        if (isBoss) {
            this.hp = BossScene.MAXHP;
        }
        this.isBoss = isBoss;
    }

    public boolean collide(Player player) {
        return Collision.collide(this, player);
    }

    /**
     * When overriding this, please call <code>super.update()</code> first!!! <br/>
     * VERY IMPORTANT
     */
    @Override
    public void update(int t) {
        // check collision
        if (collide(J.getPlayer())) {
            J.onHit();
        }

        // OOB Check
        if (enableOOBCheck && x >= U.config().w + 300 || x <= -300 || y >= U.config().h + 300
                || y <= -300) {
            J.remove(this);
        } else {

            was.update(t, x - lastx);

            lastx = x;
        }
    }

    public void draw(Batch batch) {
        TextureRegion tmp = was.getTexture();
        batch.draw(tmp, x - tmp.getRegionWidth() / 2, y - tmp.getRegionHeight() / 2);
    }

    @Override
    public Enemy setX(float x) {
        this.x = x;
        return this;
    }

    @Override
    public Enemy setY(float y) {
        this.y = y;
        return this;
    }

    @Override
    public Enemy setXY(float x, float y) {
        this.x = x;
        this.y = y;
        return this;
    }

    /**
     * Called when the enemy is on hit
     *
     * @param damage
     */
    public void onHit(float damage) {
        hp -= damage;
        if (hp <= 0) {
            onDie();
            J.remove(this);
        }
    }

    public void onDie() {

    }

    @Override
    public float getX() {
        return x;
    }

    @Override
    public float getY() {
        return y;
    }

    @Override
    public int getZIndex() {
        return -512;
    }

    @Override
    public Collision.CollisionData getCollisionData(Entity other) {
        if (other instanceof Player) {
            return collisionP;
        }
        return collisionS;
    }

}
